/****************************************************************************
Copyright (c) 2012,Otto-von-Guericke University of Magdeburg. All rights 
reserved.

This program and accompanying materials are made available under the terms of
the Eclipse Public License - v 1.0, http://www.eclipse.org/legal/epl-v10.html
 ****************************************************************************/
package index.structures;

import index.base.AIndexStructure;
import index.permutation.Result;
import index.permutation.ResultPair;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;

import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;

import testapp.Dictionary;
import testapp.Dictionary.Bundle;
import testapp.JOptionDialog;
import testapp.tapedpane.JTappedPane_Panel;

public class Permutation extends AIndexStructure {

	// TODO %*size < k

	private ArrayList<int[]> prototypes = null;

	private int numberOfProt = 4;

	private ArrayList<int[]> invperms = new ArrayList<int[]>();

	private int maxcand = 0;

	private float searchedElementsInPercent = 20;

	protected Permutation(int size, int dim, int minValue, int maxValue,
			HashMap<String, Object> map) {
		super(size, dim, minValue, maxValue, map);
		initParameter(map);

		maxcand = (int) (((double) size) * (searchedElementsInPercent / 100.0));
		// System.out.println((searchedElementsInPercent / 100.0));
		// System.out.println(maxcand);
		prototypes = new ArrayList<int[]>();

	}

	private int[] getInversePerm(int[] point, ArrayList<int[]> prototypes) {
		return getInversePerm(getPerm(point, prototypes));
	}

	private int[] getInversePerm(Integer[] perm) {
		int[] ret_inversePerm = new int[perm.length];
		for (int i = 0; i < ret_inversePerm.length; i++) {
			ret_inversePerm[perm[i]] = i;
		}
		return ret_inversePerm;
	}

	public Integer[] getPerm(int[] point, ArrayList<int[]> prototypes) {
		Double[] dist = new Double[prototypes.size()];
		Integer[] index = new Integer[prototypes.size()];

		int count = 0;
		for (int[] p : prototypes) {
			dist[count] = getMetric().calculateDistance(point, p);
			index[count] = count;

			count++;
		}

		final Double[] compDist = Arrays.copyOf(dist, dist.length);

		Arrays.sort(index, new Comparator<Integer>() {
			@Override
			public int compare(Integer o1, Integer o2) {
				return compDist[o1].compareTo(compDist[o2]);
			}
		});

		return index;
	}

	public boolean insert(int[] point) {

		if (numberOfProt < getCurrentDatasetSize()) {
			int[] invPerm_point = getInversePerm(point, prototypes);
			invperms.add(invPerm_point);
		} else
		// Save Prototyps
		if (prototypes.size() < numberOfProt) {
			prototypes.add(point);
		}// inverse of prototyps
		else if (prototypes.size() == numberOfProt) {
			for (int i = 0; i < prototypes.size(); i++) {
				int[] invPerm_point = getInversePerm(prototypes.get(i),
						prototypes);
				invperms.add(invPerm_point);
			}
			int[] invPerm_point = getInversePerm(point, prototypes);
			invperms.add(invPerm_point);
		}

		return true;
	}

	public int[] get(int[] query) {
		int[] invperm_Query = getInversePerm(query, prototypes);

		int curIdx = 0;
		for (int[] invperms_Points : invperms) {
			if (Arrays.equals(invperms_Points, invperm_Query)) {
				int[] curPoint = getDataset(curIdx);
				if (Arrays.equals(curPoint, query)) {// metric.calculateDistance(curPoint,
														// query) == 0){
					return curPoint;
				}
			}
			curIdx++;
		}
		return null;
	}

	public int[][] getKNN(int[] query, int k) {
		if (k > maxcand) {
			System.out.println("[DEBUG] Resized maxcand to k!");
			maxcand = k;
		}
		int[] invperm_Query = getInversePerm(query, prototypes);

		Result result = new Result(maxcand);

		int curIdx = 0;
		for (int[] invperms_Points : invperms) {
			double curDist = getMetric().calculateDistance(invperms_Points,
					invperm_Query);
			result.push(curIdx, curDist);
			curIdx++;
		}

		Result res_Return = new Result(k);

		for (ResultPair i : result) {
			int idx = i.getID();
			double dist = getMetric().calculateDistance(getDataset(idx), query);
			res_Return.push(idx, dist);
		}

		int[][] result_a = new int[k][query.length];

		for (int i = 0; i < k; i++) {
			result_a[i] = getDataset(res_Return.popFirst().getID());

		}

		return result_a;
	}

	public void clear() {
		prototypes = null;
		invperms = null;
		maxcand = 0;
	}

	@Override
	public int[] search(int[] datasetToSearch) {
		return get(datasetToSearch);

	}

	@Override
	public int[][] searchKNN(int[] datasetToSearch, int k) {
		return getKNN(datasetToSearch, k);
	}

	@Override
	public boolean canKNNSearch() {
		return true;
	}

	@Override
	public String getInformations() {
		return getName() + " \n"
				+ Dictionary.get(Bundle.INDEX, "percentagepoints")
				+ searchedElementsInPercent + " \n"
				+ Dictionary.get(Bundle.INDEX, "numberprototypes")
				+ numberOfProt;
	}

	public String getName() {
		return Dictionary.get(Bundle.INDEX, "permutation");
	}

	public JOptionDialog getOptionsDialog(JFrame parent) {
		return new LSH_OptionDialog(parent, this);
	}

	private class LSH_OptionDialog extends JOptionDialog {
		private static final long serialVersionUID = 7997497259063562763L;

		private JFormattedTextField field_prototypes;
		private JFormattedTextField field_SearchedElementsInPercent;
		private JTappedPane_Panel northPanel;

		public LSH_OptionDialog(JFrame parent, AIndexStructure is) {
			super(parent, is.toString());
		}

		public JPanel getNorthPanel() {
			northPanel = new JTappedPane_Panel(null) {

				/**
				 * 
				 */
				private static final long serialVersionUID = -3609572922205172977L;

				@Override
				public void refreshResource() {

				}

			};

			field_prototypes = northPanel
					.getFormtedTextFieldInstance_Integer(2);
			field_SearchedElementsInPercent = northPanel
					.getFormtedTextFieldInstance_Numeric(5, true);

			field_prototypes.setValue(numberOfProt);
			field_prototypes.setColumns(4);
			field_prototypes.setToolTipText("2-?");

			field_SearchedElementsInPercent.setValue(searchedElementsInPercent);
			field_SearchedElementsInPercent.setColumns(2);
			field_SearchedElementsInPercent.setToolTipText(Dictionary.get(
					Bundle.INDEX, "percentagepoints"));

			northPanel.addSimpleLabelAndInputRow(0,
					Dictionary.get(Bundle.INDEX, "numberprototypes") + ":",
					field_prototypes);
			northPanel.addSimpleLabelAndInputRow(1,
					Dictionary.get(Bundle.INDEX, "percentagepoints") + ":",
					field_SearchedElementsInPercent);

			return northPanel;
		}

		@Override
		public HashMap<String, Object> getOptions() {
			HashMap<String, Object> map = new HashMap<String, Object>();
			map.put("searchedElementsInPercent",
					""
							+ ((Number) field_SearchedElementsInPercent
									.getValue()).floatValue());
			map.put("numberOfProt",
					"" + ((Number) field_prototypes.getValue()).intValue());
			return map;
		}
	}

}
