/****************************************************************************
Copyright (c) 2012,Otto-von-Guericke University of Magdeburg. All rights 
reserved.

This program and accompanying materials are made available under the terms of
the Eclipse Public License - v 1.0, http://www.eclipse.org/legal/epl-v10.html
****************************************************************************/
package index.permutation;

import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;

public class Result implements Iterable<ResultPair>{
	private double dmax;
	
//			 The maximum number of items to be stored
	private int kmax;
//			
	private double MaxValue = Double.MAX_VALUE / 2;
	private TreeSet<ResultPair> pairset;
//			
	
	public int getK(){
		return this.kmax;
	}
	
//			The current items in the result set
	public int getCount() {
		return this.pairset.size();
	}
	
	// Returns the First (closer) result in the set
	public ResultPair getFirst(){
		return pairset.first();	
	}

	// Returns the Last (farthest) result in the set
	public ResultPair getLast(){
		return pairset.last();
	}
	
	// Pop First (closest) item
	public ResultPair popFirst (){
		ResultPair pair = pairset.first();
		this.pairset.remove(pair);
		return pair;
	}

	// Pop Last (farthest) item
	int cont = 0;
	public ResultPair popLast(){
		if(pairset.size()>0){
			ResultPair pair = pairset.last();

			boolean delete = pairset.remove(pair);
			if(!delete){
				System.out.println("nicht gelscht");
			}
			
			return pair;
		}
		return null;
	}


	// Constructor
	public Result (int k){
		this.kmax = k;

		this.dmax = MaxValue;
		//this.Pool = new SortedDictionary<ResultPair, int>();
		this.pairset = new TreeSet<ResultPair>(new Comparator<ResultPair>() {

			@Override
			public int compare(ResultPair o1, ResultPair o2) {
				return o1.compareTo(o2);
			}
		}); 
	}

	// The radius of the knn item (or double.MaxValue if we have less items than k)
	public double getCoveringRadius(){
		if (this.getCount() < this.kmax) {
			return this.MaxValue;
		} else {
			return this.dmax;
		}
	}
	
	public double getInnerRadius (){
		return this.pairset.last().getDist();
	}
	
	

	@Override
	public Iterator<ResultPair> iterator() {
		return this.pairset.iterator();
	}

	//  Push a docid and a distance to the result set
	public void push(int docid, double d){
		double _r = this.getCoveringRadius();
		if (d >= _r) {
			return;
		}
		ResultPair r = new ResultPair (docid, d);
		boolean removedLast = false;
		
		if (this.pairset.size() >= this.kmax) {
			removedLast = true;
			this.popLast();
		}
		this.pairset.add(r);
		
		if (removedLast) {
			this.dmax = this.getInnerRadius();
		}
	}
}
